package was7_webservice;

import ilog.rules.studio.res.server.IlrExecutionServer;
import ilog.rules.studio.res.ui.generator.impl.wizard.IlrFileGeneratorWizard;
import ilog.rules.studio.res.ui.generator.impl.wizard.page.IlrFileGeneratorWizardPage;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class WAS7ServerSetupPage extends IlrFileGeneratorWizardPage {

	// the class of the interface of the generated web service
	String wsInterface = "";
	Text wsInterfaceText;
	// the class of implementation of the interface 
	String wsImplementation = "";
	Text wsImplementationText;
	// the name of the web service
	String wsServiceName = "";
	Text wsServiceNameText;
	// the author of the decision service
	String wsAuthor = "";
	Text wsAuthorText;
	// indicated if the web service should be generated as a monitored
	// transparent decision service
	boolean wsIsMtds = false;
	Button wsIsMtdsButton;
	
	// the server configuration where the web service is deployed
	Button wsServerConfigButton;
	IlrExecutionServer conf = null;
	
	/**
	 * @param generator :  the generator used in the wizard
	 * This method passes the information collected by this page
	 * to the generator for later use.
	 */
	public void saveProperties(Was7Generator generator) {
		generator.setMtds(wsIsMtds);
		generator.setAuthor(wsAuthor.trim().length() > 0 ? wsAuthor
				: "WAS7 Web Service Generator");
		generator.setImplementationName(wsImplementation);
		generator.setInterfaceName(wsInterface);
		generator.setServer(conf);
		generator.setServiceName(wsServiceName);
	}

	protected WAS7ServerSetupPage(String pageName) {
		super(pageName);
		setTitle("Web Service Details");
		setDescription("Collect the definition of the web service");
	}

	

	@Override
	protected void addContent(Composite paramComposite) {
		String projectName = ((IlrFileGeneratorWizard) this.getWizard())
				.getJavaProjectName();
		// TODO : check that names are valid java identifiers
		// set default names:
		this.wsInterface = projectName.substring(0, 1).toUpperCase()
				+ projectName.substring(1) + "Runner";
		this.wsImplementation = this.wsInterface + "Impl";
		this.wsServiceName = projectName + "-service";
		
		// create the graphic component to collect the information
		createServiceInfoComponent(paramComposite);
	}

	public void createServiceInfoComponent(Composite parent) {
		Composite localComposite = new Composite(parent, SWT.BORDER);
		GridLayout localGridLayout = new GridLayout();
		localGridLayout.marginWidth = 0;
		localGridLayout.marginHeight = 0;
		localGridLayout.numColumns = 2;
		localGridLayout.makeColumnsEqualWidth = false;
		localComposite.setLayout(localGridLayout);
		localComposite.setLayoutData(new GridData(GridData.FILL_BOTH));

		new Label(localComposite, SWT.HORIZONTAL).setText("RES Configuration:");
		wsServerConfigButton = new Button(localComposite, SWT.PUSH);
		wsServerConfigButton.setText("Select a WebSphere RES configuration");
		wsServerConfigButton.addSelectionListener(new SelectionListener() {
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				WAS7ServerSelection was7Selection = new WAS7ServerSelection(
						Display.getCurrent().getActiveShell());
				was7Selection.open();
				if (was7Selection.getReturnCode() == 0) {
					WAS7ServerSetupPage.this.conf = was7Selection
							.getSelection();
					if (WAS7ServerSetupPage.this.conf != null)
						WAS7ServerSetupPage.this.wsServerConfigButton
								.setText(WAS7ServerSetupPage.this.conf
										.getName());
				} else {
					WAS7ServerSetupPage.this.conf = null;
					WAS7ServerSetupPage.this.wsServerConfigButton
							.setText("Select a WebSphere RES configuration");
				}
				WAS7ServerSetupPage.this
						.setPageComplete(WAS7ServerSetupPage.this
								.validatePage());

			}

			@Override
			public void widgetSelected(SelectionEvent paramSelectionEvent) {
				widgetDefaultSelected(paramSelectionEvent);
			}

		});

		new Label(localComposite, SWT.NONE).setText("Interface name:");
		this.wsInterfaceText = new Text(localComposite, SWT.BORDER);
		this.wsInterfaceText.setText(this.wsInterface);
		this.wsInterfaceText.setLayoutData(new GridData(
				GridData.GRAB_HORIZONTAL));
		this.wsInterfaceText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent paramModifyEvent) {
				WAS7ServerSetupPage.this.wsInterface = WAS7ServerSetupPage.this.wsInterfaceText
						.getText();
				WAS7ServerSetupPage.this
						.setPageComplete(WAS7ServerSetupPage.this
								.validatePage());
			}
		});
		new Label(localComposite, SWT.NONE).setText("Implementation class:");
		this.wsImplementationText = new Text(localComposite, SWT.BORDER);
		this.wsImplementationText.setLayoutData(new GridData(
				GridData.GRAB_HORIZONTAL));
		this.wsImplementationText.setText(this.wsImplementation);
		this.wsImplementationText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent paramModifyEvent) {
				WAS7ServerSetupPage.this.wsImplementation = WAS7ServerSetupPage.this.wsImplementationText
						.getText();
				WAS7ServerSetupPage.this
						.setPageComplete(WAS7ServerSetupPage.this
								.validatePage());
			}
		});
		new Label(localComposite, SWT.NONE).setText("Service name:");
		this.wsServiceNameText = new Text(localComposite, SWT.BORDER);
		this.wsServiceNameText.setLayoutData(new GridData(
				GridData.GRAB_HORIZONTAL));
		this.wsServiceNameText.setText(this.wsServiceName);
		this.wsServiceNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent paramModifyEvent) {
				WAS7ServerSetupPage.this.wsServiceName = WAS7ServerSetupPage.this.wsServiceNameText
						.getText();
				WAS7ServerSetupPage.this
						.setPageComplete(WAS7ServerSetupPage.this
								.validatePage());
			}
		});
		new Label(localComposite, SWT.NONE).setText("Author:");
		this.wsAuthorText = new Text(localComposite, SWT.BORDER);
		this.wsAuthorText.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL));
		this.wsAuthorText.setText(this.wsAuthor);
		this.wsAuthorText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent paramModifyEvent) {
				WAS7ServerSetupPage.this.wsAuthor = WAS7ServerSetupPage.this.wsAuthorText
						.getText();
			}
		});
		new Label(localComposite, SWT.NONE).setText("Generate as an MTDS:");
		this.wsIsMtdsButton = new Button(localComposite, SWT.CHECK);
		this.wsIsMtdsButton.setSelection(this.wsIsMtds);
		this.wsIsMtdsButton.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				WAS7ServerSetupPage.this.wsIsMtds = WAS7ServerSetupPage.this.wsIsMtdsButton
						.getSelection();

			}

			@Override
			public void widgetSelected(SelectionEvent e) {
				widgetDefaultSelected(e);
			}
		});

	}

	/* (non-Javadoc)
	 * @see ilog.rules.studio.res.ui.generator.impl.wizard.page.IlrFileGeneratorWizardPage#isValidatedPageAtCreation()
	 * the page is not validated at creation. It is required to select at least
	 * a server configuration. The other field are initialized with default values.
	 */
	protected boolean isValidatedPageAtCreation() {
		return false;
	}

	public void setVisible(boolean param) {
		super.setVisible(param);
	}

	/**
	 * @return true if all the information required is filled.
	 * Only the Author field can be left blank. 
	 */ 
	protected boolean validatePage() {
		return conf != null && wsInterface != null
				&& wsInterface.trim().length() > 0 && wsImplementation != null
				&& wsImplementation.trim().length() > 0
				&& wsServiceName != null && wsServiceName.trim().length() > 0;

	}

}
