package was7_webservice;

import ilog.rules.studio.res.IlrStudioResPlugin;
import ilog.rules.studio.res.generator.IlrFileGenerator;
import ilog.rules.studio.res.generator.IlrSignature;
import ilog.rules.studio.res.generator.IlrTraceFileGenerator;
import ilog.rules.studio.res.ruleapp.IlrRulesetArchiveDescriptor;
import ilog.rules.studio.res.server.IlrExecutionServer;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.util.StringUtils;
import org.eclipse.jdt.core.IJavaProject;

public class Was7Generator extends IlrTraceFileGenerator {

	protected String getTemplateDirectory() {
		String str = Activator.getPluginDir();
		Activator.getDefault().log( "Template directory:" + str +"/data" );
		if (str != null)
			return str + "/data/";
		return "";
	}

	public Was7Generator() {
		super();
	}

	private String fileHeader;
	private String interfaceName;
	private String serviceName;
	private String implementationName;
	private String author;

	protected boolean isMtds;
	private IlrExecutionServer server;


	protected String getInterfaceName() {
		return interfaceName;
	}

	protected void setInterfaceName(String interfaceName) {
		this.interfaceName = interfaceName;
	}

	protected String getServiceName() {
		return serviceName;
	}

	protected void setServiceName(String serviceName) {
		this.serviceName = serviceName;
	}

	protected String getImplementationName() {
		return implementationName;
	}

	protected void setImplementationName(String implementationName) {
		this.implementationName = implementationName;
	}

	protected String getAuthor() {
		return author;
	}

	protected void setAuthor(String author) {
		this.author = author;
	}

	protected boolean isMtds() {
		return isMtds;
	}

	protected void setMtds(boolean isMtds) {
		this.isMtds = isMtds;
	}

	protected IlrExecutionServer getServer() {
		return server;
	}

	protected void setServer(IlrExecutionServer server) {
		this.server = server;
		setServerConfiguration(server);
	}

	protected void setFileHeader(String fileHeader) {
		this.fileHeader = fileHeader;
	}

	protected String getFileHeader() {
		// TODO Auto-generated method stub
		return this.fileHeader;
	}

	protected String getPackagePrefix() {
		// TODO Auto-generated method stub
		return getRuleAppProject().getDescriptor().getName();
	}

	public void generateProjectContent(IJavaProject javaProject) {
		super.generateProjectContent(javaProject);
		// update the build path
		if (isMtds()) {
			addVariableEntryInClasspath("ILOG_EXECUTION_SERVER_HOME/lib/jrules-res-dsmbean.jar");
			addVariableEntryInClasspath("ILOG_EXECUTION_SERVER_HOME/lib/servlet-api-2.3.jar");
		}
		// generate the interface
		generateVelocityFileInSrc(getPackagePrefix(), "",
				getTemplateDirectory(), "Bean.vm",
				getInterfaceName() + ".java", ".java", "Bean.vm");
		// generate the implementation class
		generateVelocityFileInSrc(getPackagePrefix(), "",
				getTemplateDirectory(), "BeanImpl.vm", getImplementationName()
						+ ".java", ".java", "BeanImpl.vm");
		// generate the Exception class
		generateVelocityFileInSrc(getPackagePrefix(), "",
				getTemplateDirectory(), "Exception.vm",
				"RulesetExecutionException.java", ".java", "Exception.vm");
		// generate the extension class
		generateVelocityFileInSrc(getPackagePrefix(), "",
				getTemplateDirectory(), "Hook.vm", "ExecutionHook.java",
				".java", "Hook.vm");
		// generate the request result and listener class for each ruleset
		for (IlrFileGenerator.RulesetArchive ruleset : getRulesetArchives()) {
			// generating the listener if requested
			if (isMtds()) {
				HashMap<String, String> c = new HashMap<String, String>();
				c.put("ruleset", ruleset.getRulesetName());
				c.put("classname", ruleset.getRulesetClassName() + "Listener");
				c.put("description", getDescription(ruleset.getRulesetName()));
				c.put("template", "Listener.vm");
				generateVelocityFileInSrc(getPackagePrefix(), "",
						getTemplateDirectory(), "Listener.vm", ruleset
								.getRulesetClassName()
								+ "Listener.java", ".java", c);
			}
			// generating the input request
			HashMap<String, Object> c = new HashMap<String, Object>();
			c.put("classname", ruleset.getRulesetClassName() + "Request");
			c.put("imports", getImports(getInParameters(ruleset
					.getRulesetName())));
			c.put("inParameters", getInParameters(ruleset.getRulesetName()));
			c.put("template", "Request.vm");
			generateVelocityFileInSrc(getPackagePrefix(), "",
					getTemplateDirectory(), "Request.vm", ruleset
							.getRulesetClassName()
							+ "Request.java", ".java", c);
			// generating the output result
			c = new HashMap<String, Object>();
			c.put("classname", ruleset.getRulesetClassName() + "Result");
			c.put("imports", getImports(getOutParameters(ruleset
					.getRulesetName())));
			c.put("outParameters", getOutParameters(ruleset.getRulesetName()));
			c.put("template", "Result.vm");
			generateVelocityFileInSrc(getPackagePrefix(), "",
					getTemplateDirectory(), "Result.vm", ruleset
							.getRulesetClassName()
							+ "Result.java", ".java", c);
		}

		// generate resource for deployment
		generateVelocityFile("etc", "", getTemplateDirectory(), "web.vm",
				"web.xml", ".xml", "web.vm");
		generateVelocityFile("etc", "", getTemplateDirectory(),
				"application.vm", "application.xml", ".xml", "application.vm");
		// generate the build script
		generateVelocityFile("", "", getTemplateDirectory(), "build.vm",
				"build.xml", ".xml", "build.vm");
		generateVelocityFile("", "", getTemplateDirectory(),
				"buildproperties.vm", "build.properties", ".properties",
				"buildproperties.vm");
	}

	private String getDescription(String rulesetName) {

		for (IlrRulesetArchiveDescriptor archiveDescriptor : getRuleAppProject()
				.getDescriptor().getRulesetArchiveDescriptors()) {
			if (archiveDescriptor.getName().equals(rulesetName))
				return archiveDescriptor.getDescription();
		}
		return "Monitored decision service generated using " + rulesetName;
	}

	private String getRulesetVersion(String rulesetName) {
		for (IlrRulesetArchiveDescriptor archiveDescriptor : getRuleAppProject()
				.getDescriptor().getRulesetArchiveDescriptors()) {
			if (archiveDescriptor.getName().equals(rulesetName))
				return archiveDescriptor.getVersion();
		}
		return "1.0";
	}

	private List<IlrSignature> getInParameters(String rulesetName) {
		for (RulesetArchive archive : this.getRulesetArchives()) {
			if (archive.getRulesetName().equals(rulesetName)) {
				return archive.getInParameterSignatures();
			}
		}
		return new Vector<IlrSignature>();
	}

	private List<IlrSignature> getOutParameters(String rulesetName) {
		for (RulesetArchive archive : this.getRulesetArchives()) {
			if (archive.getRulesetName().equals(rulesetName)) {
				return archive.getOutParameterSignatures();
			}
		}
		return new Vector<IlrSignature>();
	}

	private List<String> getImports(List<IlrSignature> signatures) {
		Vector<String> result = new Vector<String>();

		for (IlrSignature signature : signatures) {
			String type = signature.getType();
			String objectType = signature.getObjectType();
			if (type.equals(objectType))
				result.add(type);
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see ilog.rules.studio.res.generator.IlrTraceFileGenerator#initializeContext(java.lang.Object, org.apache.velocity.VelocityContext)
	 * according to the userData we set the variables required to parse
	 * the velocity template.
	 * Some templates require more parameters in that case the userData is a HashMap
	 * for simple templates the userData is the template name
	 */
	protected VelocityContext initializeContext(Object userData,
			VelocityContext context) {

		context.put("package", getPackagePrefix());
		context.put("header", getFileHeader());
		context.put("ruleapp-name", getRuleAppProject().getDescriptor()
				.getName());
		context.put("ruleapp-version", getRuleAppProject().getDescriptor()
				.getVersion());
		context.put("rulesets", getRulesetArchives());
		context.put("exception-name", "RulesetExecutionException");
		context.put("stringUtils", new StringUtils());

		if (isGeneratedTraceFile()) {
			context.put("trace-file", "true");
			context.put("trace-attributes", getAttributes());
		}
		if (userData instanceof String) {
			String templateFile = (String) userData;
			if (templateFile.equals("Bean.vm")) {
				context.put("iclassname", getInterfaceName());
			} else if (templateFile.equals("BeanImpl.vm")) {
				context.put("iclassname", getInterfaceName());
				context.put("dsName", getServiceName());
				context.put("decisionservice", new Boolean(isMtds()));
				context.put("classname", getImplementationName());
				context.put("prefix", "");
			} else if (templateFile.equals("Hook.vm")) {
				context.put("prefix", "");
			} else if (templateFile.equals("Exception.vm")) {
				context.put("classname", "RulesetExecutionException");
			} else if (templateFile.equals("web.vm")) {
				context.put("decisionservice", new Boolean(isMtds()));
			} else if (templateFile.equals("application.vm")) {
				context.put("javaproject", getJavaProject().getProject()
						.getName());
			} else if (templateFile.equals("build.vm")) {
				context.put("javaproject", getJavaProject().getProject()
						.getName());
				context.put("xom-paths", getJavaXOMPath());
				context.put("classname", getImplementationName());
				context.put("war-xom-paths", getJavaXOMPath());
				context.put("mode",
						new Integer(getServer().getDeploymentMode()));
				context.put("build-properties", "build.properties");
				context.put("decisionservice", new Boolean(isMtds()));
			} else if (templateFile.equals("buildproperties.vm")) {
				context.put("server_home", getExecutionServerHome());
				context.put("appserverdir", getServer()
						.getInstallationDirectory());
				context.put("deploymentdir", getServer()
						.getDeploymentDirectory());
				context.put("mode",
						new Integer(getServer().getDeploymentMode()));
				if (getServer().getDeploymentMode() == 2) {
					context.put("driver", getServer()
							.getDatabaseDeploymentDriver());
					context.put("driverpath", getServer()
							.getDatabaseDeploymentDriverPath());
					context.put("url", getServer().getDatabaseDeploymentURL());
					context.put("userid", getServer()
							.getDatabaseDeploymentLogin());
					context.put("password", getServer()
							.getDatabaseDeploymentPassword());
				} else if (getServer().getDeploymentMode() == 0) {
					context.put("userid", getServer().getConsoleLogin());
					context.put("password", getServer().getConsolePassword());
					try {
						URL localObject = new URL(getServer().getConsoleURL());
						context.put("hostname", ((URL) localObject).getHost());
						context
								.put("portnumber", ((URL) localObject)
										.getPort());
					} catch (MalformedURLException localMalformedURLException) {
						IlrStudioResPlugin.getDefault().getLogger().log(
								localMalformedURLException);
					}
				}
			}
		} else if (userData instanceof HashMap) {
			HashMap<?, ?> c = (HashMap<?, ?>) userData;
			if (c.get("template").equals("Listener.vm")) {
				context.put("ruleset", c.get("ruleset"));
				context.put("classname", c.get("classname"));
				context.put("author", getAuthor());
				context.put("description", "");
				context.put("dsName", getServiceName());
				context.put("javaproject", getJavaProject().getProject()
						.getName());
				try {
					URL localObject = new URL(getServer().getConsoleURL());
					context.put("hostname", ((URL) localObject).getHost());
					context.put("portnumber", ((URL) localObject).getPort());
				} catch (MalformedURLException localMalformedURLException) {
					Activator.getDefault().log( "Unable to parse the RES Console URL.",
							localMalformedURLException);
				}
			} else if (c.get("template").equals("Request.vm")) {
				context.put("classname", c.get("classname"));
				context.put("imports", c.get("imports"));
				context.put("inParameters", c.get("inParameters"));
			} else if (c.get("template").equals("Result.vm")) {
				context.put("classname", c.get("classname"));
				context.put("imports", c.get("imports"));
				context.put("outParameters", c.get("outParameters"));
			}
		} else {
			context = super.initializeContext(userData, context);
		}
		return context;
	}

	 // removing files that are not jars.
	 // take into account XSD files in the XOM path.
	 List<String> getJavaXOMPath( )
	 {
		 List<String> input = getXOMPath();
		 Vector<String> result = new Vector();
		 for (String path : input) {
			File f = new File( path);
			// if the path does not exist don't add it.
			if ( !f.exists())
				continue;
			// files that are not jars are not listed
			if (f.isFile() & !path.endsWith(".jar"))
				continue;
			result.add(path);
		}
		return result;
	 }
}
